"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.StylableLanguageService = void 0;
const core_1 = require("@stylable/core");
const vscode_languageserver_textdocument_1 = require("vscode-languageserver-textdocument");
const vscode_languageserver_types_1 = require("vscode-languageserver-types");
const vscode_uri_1 = require("vscode-uri");
const completion_providers_1 = require("./completion-providers");
const css_service_1 = require("./css-service");
const dedupe_refs_1 = require("./dedupe-refs");
const diagnosis_1 = require("./diagnosis");
const color_provider_1 = require("./feature/color-provider");
const formatting_1 = require("./feature/formatting");
const provider_1 = require("./provider");
const provider_2 = require("./provider");
const typescript_support_1 = require("./typescript-support");
class StylableLanguageService {
    constructor({ fs, stylable }) {
        this.fs = fs;
        this.stylable = stylable;
        this.tsLanguageService = (0, typescript_support_1.typescriptSupport)(this.fs);
        this.provider = new provider_1.Provider(this.stylable, this.tsLanguageService);
        this.cssService = new css_service_1.CssService(this.fs);
    }
    getStylable() {
        return this.stylable;
    }
    getFs() {
        return this.fs;
    }
    onCompletion(filePath, offset) {
        const stylableFile = this.readStylableFile(filePath);
        if (stylableFile && stylableFile.stat.isFile()) {
            const document = vscode_languageserver_textdocument_1.TextDocument.create(vscode_uri_1.URI.file(filePath).toString(), 'stylable', stylableFile.stat.mtime.getTime(), stylableFile.content);
            const position = document.positionAt(offset);
            return this.getCompletions(document, filePath, position);
        }
        else {
            return [];
        }
    }
    onDefinition(filePath, offset) {
        const stylableFile = this.readStylableFile(filePath);
        if (stylableFile && stylableFile.stat.isFile()) {
            const doc = vscode_languageserver_textdocument_1.TextDocument.create(vscode_uri_1.URI.file(filePath).toString(), 'stylable', stylableFile.stat.mtime.getTime(), stylableFile.content);
            const res = this.provider.getDefinitionLocation(stylableFile.content, doc.positionAt(offset), filePath, this.fs);
            return res.map((loc) => vscode_languageserver_types_1.Location.create(vscode_uri_1.URI.file(loc.uri).toString(), loc.range));
        }
        return [];
    }
    onHover(filePath, offset, settings) {
        const stylableFile = this.readStylableFile(filePath);
        if (stylableFile && stylableFile.stat.isFile()) {
            const doc = vscode_languageserver_textdocument_1.TextDocument.create(vscode_uri_1.URI.file(filePath).toString(), 'stylable', stylableFile.stat.mtime.getTime(), stylableFile.content);
            return this.cssService.doHover(doc, doc.positionAt(offset), settings);
        }
        return null;
    }
    onReferences(filePath, offset) {
        const stylableFile = this.readStylableFile(filePath);
        if (stylableFile && stylableFile.stat.isFile()) {
            const doc = vscode_languageserver_textdocument_1.TextDocument.create(vscode_uri_1.URI.file(filePath).toString(), 'stylable', stylableFile.stat.mtime.getTime(), stylableFile.content);
            const position = doc.positionAt(offset);
            const refs = this.getRefs(filePath, position);
            if (refs.length) {
                return (0, dedupe_refs_1.dedupeRefs)(refs);
            }
            else {
                return (0, dedupe_refs_1.dedupeRefs)(this.cssService.findReferences(doc, position));
            }
        }
        return [];
    }
    onDocumentColor(filePath) {
        const stylableFile = this.readStylableFile(filePath);
        if (stylableFile && stylableFile.stat.isFile()) {
            const doc = vscode_languageserver_textdocument_1.TextDocument.create(vscode_uri_1.URI.file(filePath).toString(), 'stylable', stylableFile.stat.mtime.getTime(), stylableFile.content);
            return (0, color_provider_1.resolveDocumentColors)(this.stylable, this.cssService, doc, this.fs);
        }
        return [];
    }
    onColorPresentation(filePath, offset, color) {
        const stylableFile = this.readStylableFile(filePath);
        if (stylableFile && stylableFile.stat.isFile()) {
            const doc = vscode_languageserver_textdocument_1.TextDocument.create(vscode_uri_1.URI.file(filePath).toString(), 'stylable', stylableFile.stat.mtime.getTime(), stylableFile.content);
            const range = {
                start: doc.positionAt(offset.start),
                end: doc.positionAt(offset.end),
            };
            return (0, color_provider_1.getColorPresentation)(this.cssService, doc, { color, range, textDocument: doc });
        }
        return [];
    }
    onRenameRequest(filePath, offset, newName) {
        const stylableFile = this.readStylableFile(filePath);
        const edit = { changes: {} };
        if (stylableFile && stylableFile.stat.isFile()) {
            const doc = vscode_languageserver_textdocument_1.TextDocument.create(vscode_uri_1.URI.file(filePath).toString(), 'stylable', stylableFile.stat.mtime.getTime(), stylableFile.content);
            (0, provider_2.getRenameRefs)(filePath, doc.positionAt(offset), this.fs, this.stylable).forEach((ref) => {
                if (edit.changes[ref.uri]) {
                    edit.changes[ref.uri].push({ range: ref.range, newText: newName });
                }
                else {
                    edit.changes[ref.uri] = [{ range: ref.range, newText: newName }];
                }
            });
        }
        return edit;
    }
    onSignatureHelp(filePath, offset) {
        const stylableFile = this.readStylableFile(filePath);
        if (stylableFile && stylableFile.stat.isFile()) {
            const doc = vscode_languageserver_textdocument_1.TextDocument.create(vscode_uri_1.URI.file(filePath).toString(), 'stylable', stylableFile.stat.mtime.getTime(), stylableFile.content);
            const sig = this.provider.getSignatureHelp(stylableFile.content, doc.positionAt(offset), filePath, this.fs, vscode_languageserver_types_1.ParameterInformation);
            return sig;
        }
        return null;
    }
    onDocumentFormatting(filePath, options) {
        const srcText = this.fs.readFileSync(filePath, 'utf8');
        return this.getDocumentFormatting(vscode_languageserver_textdocument_1.TextDocument.create(vscode_uri_1.URI.file(filePath).toString(), 'stylable', 1, srcText), { start: 0, end: srcText.length }, (0, formatting_1.lspFormattingOptionsToJsBeautifyOptions)(options));
    }
    onDocumentRangeFormatting(filePath, offset, options) {
        const srcText = this.fs.readFileSync(filePath, 'utf8');
        return this.getDocumentFormatting(vscode_languageserver_textdocument_1.TextDocument.create(vscode_uri_1.URI.file(filePath).toString(), 'stylable', 1, srcText), offset, (0, formatting_1.lspFormattingOptionsToJsBeautifyOptions)(options));
    }
    getDocumentFormatting(doc, offset, options) {
        return (0, formatting_1.format)(doc, offset, options);
    }
    provideCompletionItemsFromSrc(src, pos, fileName) {
        return this.provider.provideCompletionItemsFromSrc(src, pos, fileName, this.fs);
    }
    getCompletions(document, filePath, position) {
        const content = document.getText();
        const stCompletions = this.provider.provideCompletionItemsFromSrc(content, {
            line: position.line,
            character: position.character,
        }, filePath, this.fs);
        const ast = (0, core_1.safeParse)(content, { from: filePath });
        const cleanDocument = this.cssService.createSanitizedDocument(ast, filePath, document.version);
        const groupedCompletions = new Map();
        for (const stComp of stCompletions) {
            const lspCompletion = vscode_languageserver_types_1.CompletionItem.create(stComp.label);
            if (!groupedCompletions.has(lspCompletion.label)) {
                groupedCompletions.set(lspCompletion.label, lspCompletion);
            }
            else {
                continue;
            }
            const textEdit = vscode_languageserver_types_1.TextEdit.replace(stComp.range
                ? stComp.range
                : new completion_providers_1.ProviderRange(new completion_providers_1.ProviderPosition(position.line, Math.max(position.character - 1, 0)), position), typeof stComp.insertText === 'string' ? stComp.insertText : stComp.insertText.source);
            lspCompletion.insertTextFormat = 2;
            lspCompletion.detail = stComp.detail;
            lspCompletion.textEdit = textEdit;
            // override sorting order to the top
            // todo: decide on actual sorting for all stylable completions
            lspCompletion.sortText = 'a';
            lspCompletion.filterText =
                typeof stComp.insertText === 'string'
                    ? stComp.insertText
                    : stComp.insertText.source;
            if (stComp.additionalCompletions) {
                lspCompletion.command = vscode_languageserver_types_1.Command.create('additional', 'editor.action.triggerSuggest');
            }
            else if (stComp.triggerSignature) {
                lspCompletion.command = vscode_languageserver_types_1.Command.create('additional', 'editor.action.triggerParameterHints');
            }
        }
        const cssCompletions = this.cssService.getCompletions(cleanDocument, position);
        for (const cssComp of cssCompletions) {
            const label = cssComp.label;
            if (!groupedCompletions.has(label)) {
                // CSS declaration property names have built in sorting
                // at-rules, rules and declaration values do not
                cssComp.sortText = cssComp.sortText || 'z';
                groupedCompletions.set(label, cssComp);
            }
        }
        return [...groupedCompletions.values()];
    }
    getDefinitionLocation(src, position, filePath) {
        const defs = this.provider.getDefinitionLocation(src, position, vscode_uri_1.URI.file(filePath).fsPath, this.fs);
        return defs.map((loc) => vscode_languageserver_types_1.Location.create(vscode_uri_1.URI.file(loc.uri).fsPath, loc.range));
    }
    getSignatureHelp(src, pos, filePath, paramInfo) {
        return this.provider.getSignatureHelp(src, pos, filePath, this.fs, paramInfo);
    }
    getRefs(filePath, position) {
        return (0, provider_2.getRefs)(filePath, position, this.fs, this.stylable);
    }
    resolveDocumentColors(document) {
        return (0, color_provider_1.resolveDocumentColors)(this.stylable, this.cssService, document, this.fs);
    }
    getColorPresentation(document, params) {
        return (0, color_provider_1.getColorPresentation)(this.cssService, document, params);
    }
    diagnose(filePath) {
        const stylableFile = this.readStylableFile(filePath);
        if (stylableFile && stylableFile.stat.isFile()) {
            return (0, diagnosis_1.createDiagnosis)(stylableFile.content, stylableFile.stat.mtime.getTime(), filePath, this.stylable, this.cssService);
        }
        return [];
    }
    readStylableFile(filePath, includeUntitled = true) {
        const supportedFile = filePath.endsWith('.st.css') || (includeUntitled && filePath.startsWith('untitled:'));
        if (!supportedFile) {
            return null;
        }
        try {
            const stat = this.fs.statSync(filePath);
            if (stat.isFile()) {
                const content = this.fs.readFileSync(filePath, 'utf8');
                return {
                    content,
                    stat,
                };
            }
        }
        catch {
            // TODO: add warning?
        }
        return null;
    }
}
exports.StylableLanguageService = StylableLanguageService;
//# sourceMappingURL=service.js.map